class RAIException(Exception):
    """RAI related exception"""

    def __init__(self, msg):
        super().__init__(msg)


class ConcurrentWriteAttemptException(RAIException):
    """Exception raised when RWM tries to submit write txn to RAI engine which already has RUNNING write txn"""

    def __init__(self, engine_name):
        super().__init__(f"'{engine_name}' has already running write transaction")


class RetryException(Exception):
    """Exception raised when retry limits are reached (timeout, max retries, etc.)"""

    def __init__(self, msg):
        super().__init__(msg)


class StepTimeOutException(Exception):
    """Exception raised when step execution time exceeds timeout"""

    def __init__(self, msg):
        super().__init__(msg)


class CommandExecutionException(Exception):
    """Exception raised when ExecuteCommand step failed"""

    def __init__(self, command, status):
        super().__init__(f"Cli command:\n{command}\nfailed with exit status '{status}'")


class RestClientException(Exception):
    """Exception raised in case of errors in RestClient"""

    def __init__(self, method, url, error):
        super().__init__(f"Rest request '{method}: {url}' failed. Exception: {error}")



from schema import Schema, Optional, And, Use
from types import MappingProxyType
from workflow.constants import CONFIGURE_SOURCES, INSTALL_MODELS, LOAD_DATA, MATERIALIZE, EXPORT, EXECUTE_COMMAND
from workflow.common import FileType


class Validator:
    schemas: dict[str, Schema]

    def __init__(self, schemas: dict[str, Schema] = MappingProxyType({})):
        self.schemas = {**STEP_SCHEMAS, **schemas}

    def validate(self, data):
        Schema({"workflow": [WorkflowStepSchema({}, self)]}).validate(data)


class WorkflowStepSchema(Schema):

    def __init__(self, schema, validator: Validator, error=None, ignore_extra_keys=False, name=None, description=None,
                 as_reference=False):
        super().__init__(schema, error, ignore_extra_keys, name, description, as_reference)
        self.validator = validator

    def validate(self, data, **kwargs):
        step_type = data.get("type", None)
        self.validator.schemas.get(step_type, Schema(step_schema, ignore_extra_keys=True)).validate(data, **kwargs)


step_schema = {
    "type": str,
    "name": str,
    Optional("engineSize"): str
}

load_data_step_schema = Schema(step_schema)

materialize_step_schema = Schema({
    **step_schema,
    "materializeJointly": bool,
    "relations": [str],
})

install_model_step_schema = Schema({
    **step_schema,
    "modelFiles": [str]
})

execute_command_step_schema = Schema({
    **step_schema,
    "command": str
})

export_step_schema = Schema({
    **step_schema,
    "exportJointly": bool,
    "dateFormat": str,
    "defaultContainer": str,
    "exports": [{
        "type": str,
        "configRelName": str,
        "relativePath": str,
        Optional("container"): str,
        Optional("snapshotBinding"): str,
        Optional("offsetByNumberOfDays"): And(int, lambda n: 0 <= n),
        Optional("metaKey"): [str]
    }]
})

configure_sources_step_schema = Schema({
    **step_schema,
    "configFiles": [str],
    "defaultContainer": str,
    "sources": [{
        "relation": str,
        "relativePath": str,
        "inputFormat": And(str, Use(str.upper), lambda s: s in FileType.__members__),
        Optional("container"): str,
        Optional("isChunkPartitioned"): bool,
        Optional("future"): bool,
        Optional("isDatePartitioned"): bool,
        Optional("loadsNumberOfDays"): And(int, lambda n: 0 <= n),
        Optional("offsetByNumberOfDays"): And(int, lambda n: 0 <= n),
        Optional("snapshotValidityDays"): And(int, lambda n: 0 <= n),
        Optional("extensions"): [str]
    }]
})

STEP_SCHEMAS = MappingProxyType(
    {
        CONFIGURE_SOURCES: configure_sources_step_schema,
        INSTALL_MODELS: install_model_step_schema,
        LOAD_DATA: load_data_step_schema,
        MATERIALIZE: materialize_step_schema,
        EXPORT: export_step_schema,
        EXECUTE_COMMAND: execute_command_step_schema
    }
)



#!/usr/bin/env python3
#
# Copyright 2023 RelationalAI, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License

from setuptools import setup, find_packages

import workflow

setup(
    author="RelationalAI, Inc.",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.9",
    ],
    description="The RelationalAI Workflow Manager for batch runs",
    install_requires=[
        "tomli==2.0.1",
        "pyyaml==6.0.1",
        "schema==0.7.5",
        "rai-sdk==0.6.18",
        "requests==2.31.0",
        "more-itertools==10.1.0",
        "azure-storage-blob==12.17.0",
        "snowflake-connector-python==3.2.0"],
    license="http://www.apache.org/licenses/LICENSE-2.0",
    long_description="The RAI Workflow Framework, which allows you to execute batch configurations, along with a "
                     "default Command-Line Interface (CLI) implementation to interact with the RAI Workflow "
                     "Framework. With this powerful combination, you can easily manage and execute batch "
                     "configurations for various tasks using the RAI database.",
    long_description_content_type="text/markdown",
    name="rai-workflow-manager",
    packages=find_packages(exclude=['test', 'cli-e2e-test']),
    entry_points={
        "console_scripts": ['rwm = cli.runner:start']
    },
    include_package_data=True,
    url="https://github.com/RelationalAI/rai-workflow-manager",
    version=workflow.__version__)
